
#include <hal/video.h>
#include <hal/xbox.h>
#include <hal/fileio.h>
#include <xboxkrnl/types.h>
#include <xboxkrnl/rtl.h>
#include <xboxkrnl/io.h>
#include <openxdk/debug.h>
#include <SDL.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <unistd.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include "mpg123.h"
#include "mpglib.h"

#define  MP3_CHUNK  4608

SDL_Event event;
SDL_AudioSpec audio_spec;
struct mpstr mp;
int ret_decode = !MP3_OK;
char input_buffer[16384];
int fd;

char remain_buffer[MP3_CHUNK];
int remain_size = 0;
int debug_index = 0;


void decode(char *buffer, int *size) {
	int remain_len, prev_len;

	if (ret_decode != MP3_OK) {
		int ret_len = read(fd, input_buffer, 16384);
		if (ret_len <= 0)
			exit(-1);
		ret_decode = decodeMP3(&mp, input_buffer, ret_len, buffer, MP3_CHUNK, size);
	}
	else
		ret_decode = decodeMP3(&mp, NULL, 0, buffer, MP3_CHUNK, size);
}


void audio_callback(void *user_data, Uint8 *stream, int len) {
	int size;
	int remain_len = len;
	Uint8 *insert = stream;

	//debugPrint("[%04d] audio_callback: len=%d\n", debug_index++, len);
	if (remain_size > 0) {
		if (remain_size >= remain_len) {
			memcpy(insert, &remain_buffer[MP3_CHUNK - remain_size], remain_len);
			remain_size -= remain_len;
			return;
		}
		else {
			memcpy(insert, &remain_buffer[MP3_CHUNK - remain_size], remain_size);
			insert += remain_size;
			remain_len -= remain_size;
		}
	}
	while (1) {
		decode(remain_buffer, &size);
		if (remain_len < size) {
			memcpy(insert, remain_buffer, remain_len);
			insert += remain_len;
			remain_size = MP3_CHUNK - remain_len;
			break;
		}
		else {
			memcpy(insert, remain_buffer, size);
			insert += size;
			remain_len -= size;
			remain_size = 0;
			if (remain_len == 0)
				break;
		}
	}
}


void XBoxStartup(void) {
	ANSI_STRING a1, a2;

	RtlInitAnsiString(&a1, "\\??\\D:");
	RtlInitAnsiString(&a2, "\\Device\\CdRom0");
	IoCreateSymbolicLink(&a1, &a2);
	
	SDL_Init(SDL_INIT_VIDEO | SDL_INIT_AUDIO);
	SDL_SetVideoMode(640, 480, 24, SDL_SWSURFACE | SDL_HWSURFACE);
	audio_spec.freq = 44100;
	audio_spec.format = AUDIO_S16;
	audio_spec.channels = 2;
	audio_spec.samples = 16384;
	audio_spec.callback = audio_callback;
	audio_spec.userdata = NULL;
	if (SDL_OpenAudio(&audio_spec, NULL) < 0) {
		debugPrint("Couldn't open audio: %s\n", SDL_GetError());
		while (1)
			;
	}
	if ((fd = open("./example.mp3", O_RDONLY)) < 0) {
		debugPrint("Couldn't open file example.mp3\n");
		while (1)
			;
	}
	debugPrint("Init MP3\n");
	InitMP3(&mp);
	debugPrint("Playing...\n");
	SDL_PauseAudio(0);
	while (1) {
		/*
		SDL_WaitEvent(&event);
		if (event.type == SDL_KEYDOWN) {
			if (event.key.keysym.sym == SDLK_ESCAPE)
				break;
		}
		*/
	}
	/* SDL_Quit(); */
}
